################################################################################
#
# df3.py
#
# Copyright (C) 2005 Mike Kost <contact@povray.tashcorp.net>
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#
#
################################################################################

import struct
import os
import stat

# -+-+-+- Start df3 Class -+-+-+-

class df3:
    __version__ = '0.1'

    def __init__(self, x=1, y=1, z=1):
        self.create(x, y, z)

    def sizeX(self):
        try:
            return (len(self.voxel))
        except:
            print "Improperly formatted voxel array";
            return (0)

    def sizeY(self):
        try:
            return (len(self.voxel[0]))
        except:
            print "Improperly formatted voxel array";
            return (0)

    def sizeZ(self):
        try:
            return (len(self.voxel[0][0]))
        except:
            print "Improperly formatted voxel array";
            return (0)

    def size(self):
        tmp = []
        tmp.append(self.sizeX())
        tmp.append(self.sizeY())
        tmp.append(self.sizeZ())
        return tmp

    def create(self, x, y, z):
        self.voxel = []

        for i in range(x):
            self.voxel.append([])
            for j in range(y):
                self.voxel[i].append([])
                for k in range(z):
                    self.voxel[i][j].append(0)

    def exportDF3(self, file):
        try:
            x = self.sizeX()
            y = self.sizeY()
            z = self.sizeZ()
            i = self.voxel[x-1][y-1][z-1]
        except:
            print "Improperly formatted voxel array";
            return

        try:
            f = open(file, 'wb');
        except:
            print "Could not open " + file + " for write";
            return

        f.write(struct.pack('B',(x & (0xff << 8)) >> 8));
        f.write(struct.pack('B',(x & (0xff << 0)) >> 0));

        f.write(struct.pack('B',(y & (0xff << 8)) >> 8));
        f.write(struct.pack('B',(y & (0xff << 0)) >> 0));

        f.write(struct.pack('B',(z & (0xff << 8)) >> 8));
        f.write(struct.pack('B',(z & (0xff << 0)) >> 0));

        vMin = 0
        vMax = 0

        for i in range(x):
            for j in range(y):
                for k in range(z):
                    vMin = min(self.voxel[i][j][k], vMin)
                    vMax = max(self.voxel[i][j][k], vMax)

        if (vMin < 0):
            print "Voxel cannot be negative"
            return

        for k in range(z):
            for j in range(y):
                for i in range(x):
                    if (vMax >= (1 << 16)): # 32-bit
                        f.write(struct.pack('B',(self.voxel[i][j][k] & (0xff << 24)) >> 24));
                        f.write(struct.pack('B',(self.voxel[i][j][k] & (0xff << 16)) >> 16));

                    if (vMax >= (1 << 8)): # 16-bit
                        f.write(struct.pack('B',(self.voxel[i][j][k] & (0xff << 8)) >> 8));

                    f.write(struct.pack('B',(self.voxel[i][j][k] & (0xff << 0)) >> 0));

    def importDF3(self, file):
        try:
            f = open(file, 'rb');
            size = os.stat(file)[stat.ST_SIZE]

        except:
            print "Could not open " + file + " for read";
            return []

        x = (0 | struct.unpack('B', f.read(1) )[0] << 8)
        x = (x | struct.unpack('B', f.read(1) )[0] << 0)

        y = (0 | struct.unpack('B', f.read(1) )[0] << 8)
        y = (y | struct.unpack('B', f.read(1) )[0] << 0)

        z = (0 | struct.unpack('B', f.read(1) )[0] << 8)
        z = (z | struct.unpack('B', f.read(1) )[0] << 0)

        self.create(x, y, z)

        size = size-6
        if (size == x*y*z):
            format = 8
        if (size == 2*x*y*z):
            format = 16
        if (size == 4*x*y*z):
            format = 32

        for k in range(z):
            for j in range(y):
                for i in range(x):
                    temp = 0
                    if (format == 32):
                        temp = (temp | struct.unpack('B', f.read(1) )[0] << 24)
                        temp = (temp | struct.unpack('B', f.read(1) )[0] << 16)
                    if (format >= 16):
                        temp = (temp | struct.unpack('B', f.read(1) )[0] << 8)

                    temp = (temp | struct.unpack('B', f.read(1) )[0] << 0)
                    self.voxel[i][j][k] = temp

        return self

# -=-=-=- End df3 Class -=-=-=-

if __name__ == '__main__':
    localX = 20
    localY = 30
    localZ = 40
    # Generate an output
    temp = df3(localX, localY, localZ)

    for i in range(localX):
        for j in range(localY):
            for k in range(localZ):
                if (i >= (localX/2)):
                    temp.voxel[i][j][k] = 255

    temp.exportDF3('temp.df3')

    # Import
    temp2 = df3().importDF3('temp.df3')

    # Compare
    print temp2.size()
    for i in range(temp2.sizeX()):
        for j in range(temp2.sizeY()):
            for k in range(temp2.sizeZ()):
                print i, j, k, temp.voxel[i][j][k], temp2.voxel[i][j][k]
